# -*- coding: utf-8 -*-
"""The entry point to the ``format_pipfile`` package."""

from __future__ import absolute_import

if __name__ == "__main__":
    from format_pipfile.cli import main

    main()
