checks:
  python:
    code_rating: true
    duplicate_code: true
    classes_valid_slots: true

build:
  environment:
    python: 3.6
  nodes:
    analysis:
      tests:
        override:
          - py-scrutinizer-run
    tests:
      tests:
        override:
          - true
filter:
  excluded_paths:
    - "*/test/*"
  dependency_paths:
    - "lib/*"
tools:
  external_code_coverage:
{% set items = [] %}
{% for env, config in environments|dictsort if config.cover %}
{% do items.append(env) %}
{% endfor %}
    # {{ items|join(",") }}
    runs: {{ items|length }}
    # timeout in seconds, default 5 minutes
    timeout: {{ items|length * 90 }}
